@@ -1,22 +1,19 @@
-package WebDriver::Tiny 0.003;
+package WebDriver::Tiny 0.004;
 
-use 5.014;
+use 5.020;
+use feature 'postderef';
 use warnings;
+no  warnings 'experimental::postderef';
 
 # Allow "cute" $drv->('selector') syntax.
 #
-# Caching the closure is around twice as fast, weaken to avoid $self
-# circularly referencing $self.
-use overload
-    fallback => 1,
-    '&{}'    => sub {
-        $_[0][4] ||= do {
-            Scalar::Util::weaken( my $self = $_[0] );
-
-            sub { $self->find(@_) };
-        };
-    };
+# $self->[4] contains a sub ref that closes over a weakened $self that calls
+# find, i.e. sub { $weak_self->find(@_) }. This sub ref is returned when $self
+# is invoked as a sub ref thanks to the magic of overloading. We weaken to
+# avoid a memory leak. The closure is built in new().
+use overload fallback => 1, '&{}' => sub { $_[0][4] };
 
+use Carp 1.25 ();
 use HTTP::Tiny;
 use JSON::PP ();
 use WebDriver::Tiny::Elements;
@@ -62,14 +59,20 @@ sub import {
     charnames->import( ':alias' => $chars );
 }
 
+# We're a blessed arrayref (for speed) of the following parts:
+#
+# $self = [
+#   0: HTTP::Tiny instance,
+#   1: URL of the WebDriver daemon,
+#   2: Base URL which schemeless get calls are based off,
+#   3: The capabilities of the WebDriver daemon,
+#   4: Cached closure of $self for ->() syntax,
+# ]
 sub new {
     my ( $class, %args ) = @_;
 
-    unless ( exists $args{port} ) {
-        require Carp;
-
-        Carp::croak qq/$class - Missing required parameter "port"/;
-    }
+    Carp::croak qq/$class - Missing required parameter "port"/
+        unless exists $args{port};
 
     $args{host} //= 'localhost';
     $args{path} //= '';
@@ -90,29 +93,72 @@ sub new {
     $self->[3] = $reply->{value};
 
     # Numify bool objects, saves memory.
-    $_ += 0 for grep ref eq 'JSON::PP::Boolean', values %{ $self->[3] };
+    $_ += 0 for grep ref eq 'JSON::PP::Boolean', values $self->[3]->%*;
+
+    # See the overloading at the top of the file for details.
+    Scalar::Util::weaken( my $weak_self = $self );
+    $self->[4] = sub { $weak_self->find(@_) };
 
     $self;
 }
 
 sub capabilities { $_[0][3] }
 
-sub page_ids { $_[0]->_req( GET  => '/window_handles' )->{value} }
-sub source   { $_[0]->_req( GET  => '/source'         )->{value} }
-sub title    { $_[0]->_req( GET  => '/title'          )->{value} }
-sub url      { $_[0]->_req( GET  => '/url'            )->{value} }
+sub page_ids { $_[0]->_req( GET => '/window_handles' )->{value} }
+sub source   { $_[0]->_req( GET => '/source'         )->{value} }
+sub title    { $_[0]->_req( GET => '/title'          )->{value} }
+sub url      { $_[0]->_req( GET => '/url'            )->{value} }
 
 sub back       { $_[0]->_req( POST   => '/back'    ); $_[0] }
 sub close_page { $_[0]->_req( DELETE => '/window'  ); $_[0] }
 sub forward    { $_[0]->_req( POST   => '/forward' ); $_[0] }
 sub refresh    { $_[0]->_req( POST   => '/refresh' ); $_[0] }
 
+sub status {
+    # /status is the only path without the session prefix, so surpress it.
+    local $_[0][1] = substr $_[0][1], 0, rindex $_[0][1], '/session/';
+
+    $_[0]->_req( GET => '/status' )->{value};
+}
+
+sub storage {
+    my ( $self, $key, $value ) = @_;
+
+    # Set a key.
+    if ( @_ == 3 ) {
+        my $ret = $self->_req(
+            POST => '/local_storage', { key => $key, value => $value } );
+
+        Carp::croak $ret->{value}{message} if $ret->{value}{message};
+
+        $self;
+    }
+    # Get a key.
+    elsif ( @_ == 2 ) {
+        $self->_req( GET => "/local_storage/key/$key" )->{value};
+    }
+    # List all keys.
+    else {
+        my @keys = $self->_req( GET => '/local_storage' )->{value}->@*;
+
+        return @keys if wantarray;
+
+        +{ map {
+            $_ => $self->_req( GET => "/local_storage/key/$_" )->{value}
+        } @keys };
+    }
+}
+
 sub accept_alert {
     $_[0]->_req( POST => '/accept_alert' ) if $_[0][3]{handlesAlerts};
 
     $_[0];
 }
 
+sub alert_text {
+    $_[0][3]{handlesAlerts} ? $_[0]->_req( GET => '/alert_text' )->{value} : ();
+}
+
 sub dismiss_alert {
     $_[0]->_req( POST => '/dismiss_alert' ) if $_[0][3]{handlesAlerts};
 
@@ -132,13 +178,8 @@ sub base_url {
 sub cookie {
     my ( $self, $name, $value, @args ) = @_;
 
-    if ( @_ == 2 ) {
-        my $cookie = $self->_req( GET => "/cookie/$name" )->{value};
-
-        # FIXME PhantomJS returns all cookies.
-        return ref $cookie eq 'ARRAY'
-            ? grep $_->{name} eq $name, @$cookie : $cookie;
-    }
+    # GET /cookie/{name} isn't supported by ChromeDriver, so get all.
+    return $self->cookies->{$name} if @_ == 2;
 
     $self->_req( POST => '/cookie',
         { cookie => { name => $name, value => $value, @args } } );
@@ -160,10 +201,12 @@ sub cookie_delete {
 }
 
 sub cookies {
-    +{
-        map { $_->{name} => $_ }
-        @{ $_[0]->_req( GET => '/cookie' )->{value} // [] }
-    };
+    my @cookies = @{ $_[0]->_req( GET => '/cookie' )->{value} // [] };
+
+    # Map the incorrect key to the correct key.
+    $_->{httpOnly} //= delete $_->{httponly} for @cookies;
+
+    +{ map { $_->{name} => $_ } @cookies };
 }
 
 # NOTE This method can be called from a driver or a collection of elements.
@@ -191,7 +234,7 @@ sub find {
             { using => $method, value => $selector },
         );
 
-        @ids = map $_->{ELEMENT}, @{ $reply->{value} };
+        @ids = map $_->{ELEMENT}, $reply->{value}->@*;
 
         # FIXME This'll break when called on elems->find(), this always need
         # to be $drv NOT $self.
@@ -204,11 +247,8 @@ sub find {
         select undef, undef, undef, $args{sleep} // .1;
     }
 
-    if ( !@ids && !exists $args{dies} && !$args{dies} ) {
-        require Carp;
-
-        Carp::croak ref $self, qq/->find failed for $method = "$_[1]"/;
-    }
+    Carp::croak ref $self, qq/->find failed for $method = "$_[1]"/
+        if !@ids && !exists $args{dies} && !$args{dies};
 
     # FIXME
     $self = $self->[0] if ref $self eq 'WebDriver::Tiny::Elements';
@@ -328,8 +368,6 @@ sub _req {
             JSON::PP::decode_json($msg)->{errorMessage}
         };
 
-        require Carp;
-
         Carp::croak ref $self, ' - ', $error // $reply->{content};
     }
 
