@@ -1,18 +1,20 @@
-package WebDriver::Tiny::Elements 0.003;
+package WebDriver::Tiny::Elements 0.004;
 
-use 5.014;
+use 5.020;
+use feature 'postderef';
 use warnings;
+no  warnings 'experimental::postderef';
 
 # Manip
-sub append { bless [ @{ +shift }, map @$_[ 1.. $#$_ ], @_ ] }
-sub first  { bless [ @{ $_[0] }[ 0,  1 ] ] }
-sub last   { bless [ @{ $_[0] }[ 0, -1 ] ] }
+sub append { bless [ shift->@*, map @$_[ 1.. $#$_ ], @_ ] }
+sub first  { bless [ $_[0]->@[ 0,  1 ] ] }
+sub last   { bless [ $_[0]->@[ 0, -1 ] ] }
 sub size   { $#{ $_[0] } }
-sub slice  { my ( $drv, @ids ) = @{ +shift }; bless [ $drv, @ids[@_] ] }
-sub split  { my ( $drv, @ids ) = @{ $_[0] }; map { bless [ $drv, $_ ] } @ids }
+sub slice  { my ( $drv, @ids ) = shift->@*; bless [ $drv, @ids[@_] ] }
+sub split  { my ( $drv, @ids ) = $_[0]->@*; map { bless [ $drv, $_ ] } @ids }
 
 sub uniq {
-    my ( $drv, @ids ) = @{ $_[0] };
+    my ( $drv, @ids ) = $_[0]->@*;
 
     bless [ $drv, keys %{ { map { $_ => undef } @ids } } ];
 }
@@ -30,6 +32,12 @@ sub selected { $_[0]->_req( GET => '/selected'  )->{value} }
 sub tag      { $_[0]->_req( GET => '/name'      )->{value} }
 sub visible  { $_[0]->_req( GET => '/displayed' )->{value} }
 
+# Slice off just 'x' & 'y' as various backends like to supply other junk too.
+sub location { +{ $_[0]->_req( GET => '/location' )->{value}->%{'x', 'y'} } }
+sub location_in_view {
+    +{ $_[0]->_req( GET => '/location_in_view' )->{value}->%{'x', 'y'} };
+}
+
 *find       = \&WebDriver::Tiny::find;
 *screenshot = \&WebDriver::Tiny::screenshot;
 
@@ -104,11 +112,11 @@ sub submit {
         return [ click, keys ];
 JS
 
-    $drv->_req( POST => "/element/$_->{ELEMENT}/click" ) for @{ $elems->[0] };
+    $drv->_req( POST => "/element/$_->{ELEMENT}/click" ) for $elems->[0]->@*;
 
     $drv->_req(
         POST => "/element/$_->[0]{ELEMENT}/value", { value => [ $_->[1] ] },
-    ) for @{ $elems->[1] };
+    ) for $elems->[1]->@*;
 
     $self->_req( POST => '/submit' );
 
@@ -116,7 +124,7 @@ JS
 }
 
 sub text {
-    my ( $drv, @ids ) = @{ $_[0] };
+    my ( $drv, @ids ) = $_[0]->@*;
 
     join ' ', map $drv->_req( GET => "/element/$_/text" )->{value}, @ids;
 }
